#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

int main(int argc, char *argv[]) {
	FILE *log;
	int i;
	int child;
	struct timeval tv_start;
	struct timeval tv_end;
	struct rusage rusage;
	int status;
	int exitstatus;
	char command[4096];
	int cmdlen = 0;

	double utime;
	double stime;
	double etime;

	double seconds;

	if (argc < 3) {
		fprintf(stderr, "%s log ...\n", argv[0]);
		exit(1);
	}

	log = fopen(argv[1], "a");
	if (!log) {
		fprintf(stderr, "fopen(%s): %s\n", argv[1], strerror(errno));
		exit(1);
	}

	command[0] = '\0';
	for (i = 2; i < argc; i++) {
		int l = strlen(argv[i]);
		if (strcspn(argv[i], " \"\'\\") != strlen(argv[i])) {
			char *tmp;
			int j, k;

			tmp = (char *)malloc(l * 2);
			for (k = j = 0; j < l; j++) {
				switch(argv[i][j]) {
				case '\\':
				case '\"':
					tmp[k++] = '\\';
				default:
					tmp[k++] = argv[i][j];
					break;
				}
			}
			tmp[k] = '\0';
			cmdlen += snprintf(command + cmdlen, sizeof(command) - cmdlen, "\"%s\"", tmp);
			free(tmp);
		} else {
			cmdlen += snprintf(command + cmdlen, sizeof(command) - cmdlen, "%s", argv[i]);
		}
		if (i < argc - 1) {
			command[cmdlen++] = ' ';
		}
		command[cmdlen] = '\0';
	}
	fprintf(log, "command = %s\n", command);
	fflush(log);

	gettimeofday(&tv_start, NULL);

	switch(child = fork()) {
	case 0:
		fclose(log);
		execvp(argv[2], &argv[2]);
		fprintf(stderr, "execvp(%s): %s\n", argv[2], strerror(errno));
		exit(127);
		break;
	case -1:
		perror("fork()");
		exit(1);
		break;
	}
	while(wait3(&status, 0, &rusage) != child);
	gettimeofday(&tv_end, NULL);

	if (WIFEXITED(status)) {
		exitstatus = WEXITSTATUS(status);
	} else if (WIFSIGNALED(status)) {
		fprintf(stderr, "Terminated by signal %d\n", WTERMSIG(status));
		exitstatus = 128 + WTERMSIG(status);
	} else {
		fprintf(stderr, "aptime: Process neither exited, or was terminated by a signal!\n");
		exitstatus = 1;
	}

	fprintf(log, "status = %d\n", exitstatus);

	seconds = (double)tv_end.tv_sec - (double)tv_start.tv_sec;
	seconds += (double)((double)((double)tv_end.tv_usec - (double)tv_start.tv_usec) / (double)1000000);
	fprintf(log, "elapsed = %.06lf\n", seconds);

	utime = (double)rusage.ru_utime.tv_sec + ((double)rusage.ru_utime.tv_usec / (double)1000000);
	stime = (double)rusage.ru_stime.tv_sec + ((double)rusage.ru_stime.tv_usec / (double)1000000);
	etime = ((tv_end.tv_sec - tv_start.tv_sec) + ((double)(tv_end.tv_usec - tv_start.tv_usec) / (double)1000000));
	fprintf(log, "user = %.06lf\n", utime);
	fprintf(log, "sys = %.06lf\n", stime);


	exit(exitstatus);
}
